// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved

#pragma once

// Modify the following defines if you have to target a platform prior to the ones specified below.
// Refer to MSDN for the latest info on corresponding values for different platforms.
#ifndef WINVER              // Allow use of features specific to Windows 7 or later.
#define WINVER 0x0700       // Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef _WIN32_WINNT        // Allow use of features specific to Windows 7 or later.
#define _WIN32_WINNT 0x0700 // Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef UNICODE
#define UNICODE
#endif

#define WIN32_LEAN_AND_MEAN     // Exclude rarely-used stuff from Windows headers
// Windows Header Files:
#include <windows.h>

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <wchar.h>

#include <d2d1.h>
#include <d2d1helper.h>
#include <dwrite.h>
#include <wincodec.h>
#include <dwmapi.h>

#include "Animation.h"

/******************************************************************
*                                                                 *
*  Macros                                                         *
*                                                                 *
******************************************************************/

template<class Interface>
inline void
SafeRelease(
    Interface **ppInterfaceToRelease
    )
{
    if (*ppInterfaceToRelease != NULL)
    {
        (*ppInterfaceToRelease)->Release();

        (*ppInterfaceToRelease) = NULL;
    }
}

#ifndef Assert
#if defined( DEBUG ) || defined( _DEBUG )
#define Assert(b) if (!(b)) {OutputDebugStringA("Assert: " #b "\n");}
#else
#define Assert(b)
#endif //DEBUG || _DEBUG
#endif

#ifndef HINST_THISCOMPONENT
EXTERN_C IMAGE_DOS_HEADER __ImageBase;
#define HINST_THISCOMPONENT ((HINSTANCE)&__ImageBase)
#endif


/******************************************************************
*                                                                 *
*  DemoApp                                                        *
*                                                                 *
******************************************************************/

class DemoApp
{
public:
    DemoApp();
    ~DemoApp();

    HRESULT Initialize();

    void RunMessageLoop();

private:
    HRESULT CreateDeviceIndependentResources();
    HRESULT CreateDeviceResources();
    void DiscardDeviceResources();

    HRESULT OnRender();

	HRESULT LoadResourceBitmap(
        ID2D1RenderTarget *pRT,
        IWICImagingFactory *pIWICFactory,
        PCWSTR resourceName,
        PCWSTR resourceType,
        ID2D1Bitmap **ppBitmap
        );

    void OnResize(
        UINT width,
        UINT height
        );

    void OnTimer();

    static LRESULT CALLBACK WndProc(
        HWND hWnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam
        );

private:
    HWND m_hwnd;
    ID2D1Factory *m_pD2DFactory;
    ID2D1HwndRenderTarget *m_pRT;
    ID2D1PathGeometry *m_pPathGeometry;
	ID2D1PathGeometry *m_pPathGeometry2;
	ID2D1PathGeometry *m_pPathGeometry3;


    ID2D1SolidColorBrush *m_pRedBrush;
	ID2D1SolidColorBrush *m_pYellowBrush;
	ID2D1SolidColorBrush *m_pGrayBrush;
	ID2D1StrokeStyle *m_pStrokeStyleDash;

	//DWrite
    IDWriteFactory *m_pDWriteFactory;
    IDWriteTextFormat *m_pTextFormat;

	LinearAnimation<float> m_Animation;
	EaseInOutExponentialAnimation<float> m_Animation2;
	EaseOutExponentialAnimation<float> m_Animation3;
//  EaseInOutExponentialAnimation<float> m_Animation;
//	EaseInExponentialAnimation<float> m_Animation;
//	EaseOutExponentialAnimation<float> m_Animation;

	// WIC and Bitmaps
    IWICImagingFactory *m_pWICFactory;
    ID2D1Bitmap *m_pBackgroundBitmap;
	ID2D1Bitmap *m_pBackgroundBitmap2;
	ID2D1Bitmap *m_pBackgroundBitmap3;
	ID2D1Bitmap *m_pObject1Bitmap;
	ID2D1Bitmap *m_pStarBitmap;
	ID2D1Bitmap *m_pStarBitmap2;

	ID2D1Bitmap *m_pObject1Bitmap_mask;
	ID2D1Bitmap *m_pBackgroundBitmap2_mask;
	ID2D1Bitmap *m_pBackgroundBitmap3_mask;

	

    // Bitmap brushes
    ID2D1BitmapBrush *m_pBackgroundBitmapBrush;
	ID2D1BitmapBrush *m_pBackgroundBitmapBrush2;
	ID2D1BitmapBrush *m_pBackgroundBitmapBrush3;
	ID2D1BitmapBrush *m_pObject1BitmapBrush;
	ID2D1BitmapBrush *m_pStarBitmapBrush;
	ID2D1BitmapBrush *m_pStarBitmapBrush2;

	ID2D1BitmapBrush *m_pObject1BitmapBrush_mask;
	ID2D1BitmapBrush *m_pBackgroundBitmapBrush2_mask;
	ID2D1BitmapBrush *m_pBackgroundBitmapBrush3_mask;

    DWM_TIMING_INFO m_DwmTimingInfo;
	DWM_TIMING_INFO m_DwmTimingInfo2;
	DWM_TIMING_INFO m_DwmTimingInfo3;
};


